/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.example.warehouse.manager.util;

import org.bluestemsoftware.open.eoa.engine.spring.SpringEngine;
import org.bluestemsoftware.open.eoa.example.warehouse.manager.service.Service;
import org.bluestemsoftware.specification.eoa.application.spring.MyApplicationContext;
import org.springframework.context.ApplicationContext;

public class ServiceLocator {

    private static ApplicationContext applicationContext;

    /**
     * Implements service locator pattern. Used by application objects to retrieve
     * <code>Service</code> by service type, i.e. in situations where IOC is not practical.
     * Demonstrates how access to root application context may be obtained.
     * 
     * @param serviceType
     *        by convention simple name of service type MUST match bean name.
     * 
     * @return
     */
    public static <T extends Service> T locateService(Class<T> serviceType) {
        String beanName = serviceType.getSimpleName();
        return serviceType.cast(getApplicationContext().getBean(beanName));
    }
    
    private static ApplicationContext getApplicationContext() {
        if (applicationContext == null) {
            SpringEngine se = (SpringEngine)MyApplicationContext.getApplication();
            applicationContext = se.getApplicationContext();
        }
        return applicationContext;
    }

}
